/*
	main.c
	By Tuxgasy <tuxgasy@tuxfamily.org>
	
	This file is part of Foobar.
	
	You may redistribute it and/or modify it under the terms of the
	GNU General Public License, as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later version.
	
	Lda is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Lda; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "ldaXML.h"
#include "ldaChainee.h"
#include "main.h"

int main(int argc, char *argv[]){
	xmlDocPtr ptrFic;
	listeArticles *ptrLdaChainee;
	char *commande;
	char *cpyFic;
	char *ficListeXML;
	enum bool onContinu;

	printf("=== lda v%s ===\n\n", VERSION_LDA);
	
	if(!argv[1]){
		fprintf(stderr, "lda: opérande(s) manquante(s).\n");
		fprintf(stderr, "Syntaxe : lda <FichierDeSauvegarde>.\n");
		return EXIT_FAILURE;
	}
	
	ficListeXML = malloc(sizeof(char)*30);
	sprintf(ficListeXML, "%s%s%s", getenv("HOME"), DIR_HOME_LDA, FIC_LISTE_XML);
	
	if(!is_file(ficListeXML)){
		cpyFic = malloc(sizeof(char)*100);
		sprintf(cpyFic, "%s%s", getenv("HOME"), DIR_HOME_LDA);
		mkdir(cpyFic, 0755);
		sprintf(cpyFic, "cp /usr/share/lda/liste.xml.default %s", ficListeXML);
		system(cpyFic);
		free(cpyFic);
	}
	
	ptrFic = initListe(ficListeXML);
	ptrLdaChainee = initListeArticles();
	
	if(ptrFic == NULL){
		return EXIT_FAILURE;
	}
	
	commande = malloc(sizeof(char)*20);
	sprintf(commande, START_LDA);
	onContinu = VRAI;
	while(onContinu == VRAI){
		printf("> ");
		scanf("%s", commande);
		if(strcmp(commande, HELP_LDA) == 0){
			help_cmd();
		}
		else if(strcmp(commande, SHOWALL_LDA) == 0){
			ShowAll(ptrFic);
		}
		else if(strcmp(commande, SHOWCAT_LDA) == 0){
			ShowCat(ptrFic);
		}
		else if(strcmp(commande, SHOWSSCAT_LDA) == 0){
			ShowSSCat(ptrFic);
		}
		else if(strcmp(commande, SHOWART_LDA) == 0){
			ShowArt(ptrFic);
		}
		else if(strcmp(commande, ADDART_LDA) == 0){
			ptrLdaChainee = AddArt(ptrFic, ptrLdaChainee);
		}
		else if(strcmp(commande, SHOWMYLIST_LDA) == 0){
			ShowMyList(ptrFic, ptrLdaChainee);
		}
		else if(strcmp(commande, SAVE_LDA) == 0){
			SaveMyList(ptrFic, ptrLdaChainee, argv[1]);
		}
		else if(strcmp(commande, EXIT_LDA) == 0){
			onContinu = FAUX;
			printf("Bye! ;)\n");
		}
		else{
			fprintf(stderr, "%s: commande inconnue\n", commande);
			fprintf(stderr, "Pour en savoir davantage, faites: « help ».\n");
		}
	}

	/* Liberation de la mémoire */
	free(commande);
	free(ficListeXML);
	xmlFreeDoc(ptrFic);
	freeLdaChainee(ptrLdaChainee);

	return EXIT_SUCCESS;
}

void help_cmd(){
	printf("%s : Affiche l'aide sur les commandes\n", HELP_LDA);
	
	printf("%s : Affiche tous les contenus de la liste (Catégories/Sous catégories/Articles)\n", SHOWALL_LDA);
	printf("%s : Affiche la liste des catégories\n", SHOWCAT_LDA);
	printf("%s : Affiche la liste des sous catégories pour une catégorie donnée\n", SHOWSSCAT_LDA);
	printf("%s : Affiche la liste des articles pour une catégorie et une sous catégorie données\n", SHOWART_LDA);
	
	printf("%s : Ajoute un article dans sa liste d'achat\n", ADDART_LDA);
	printf("%s : Affiche le contenu de sa liste d'achat\n", SHOWMYLIST_LDA);
	printf("%s : Sauvegarde sa liste d'achat dans le fichier passé en argument\n", SAVE_LDA);
	
	printf("%s : Quitte le programme\n", EXIT_LDA);
	
	printf("\n");
}

void ShowAll(xmlDocPtr ptrFic){
	char *ptrCategory, *ptrSSCategory, *ptrArticleName, *ptrArticlePrice;
	int cpt1, cpt2, cpt3;
	
	printf("Liste des toutes les articles:\n");
	cpt1 = 1;
	do{
		ptrCategory = getCategoryById(ptrFic, cpt1);
		if(strcmp(ptrCategory, "FIN") != 0){
			printf("%d : %s\n", cpt1, ptrCategory);
			
			cpt2 = 1;
			do{
				ptrSSCategory = getSousCategoryByCategory(ptrFic, cpt1, cpt2);
				if(strcmp(ptrSSCategory, "FIN") != 0){
					printf("\t%d-%d : %s\n", cpt1, cpt2, ptrSSCategory);
					
					cpt3 = 1;
					do{
						ptrArticleName = getArticleNameBySousCategory(ptrFic, cpt1, cpt2, cpt3);
						if(strcmp(ptrArticleName, "FIN") != 0){
							ptrArticlePrice = getArticlePriceBySousCategory(ptrFic, cpt1, cpt2, cpt3);
							printf("\t\t%d-%d-%d : %s = %s €\n", cpt1, cpt2, cpt3, ptrArticleName, ptrArticlePrice);
						}
						cpt3++;
					}while(strcmp(ptrArticleName, "FIN") != 0);
				}
				cpt2++;
			}while(strcmp(ptrSSCategory, "FIN") != 0);
		}	
		cpt1++;
	}while(strcmp(ptrCategory, "FIN") != 0);
	
	printf("\n");
	
	/* Liberation de la mémoire */
	free(ptrCategory);
	free(ptrSSCategory);
	free(ptrArticleName);
	free(ptrArticlePrice);
}

void ShowCat(xmlDocPtr ptrFic){
	char *ptrCategory;
	int cpt;
	
	printf("Liste des category:\n");
	cpt = 1;
	do{
		ptrCategory = getCategoryById(ptrFic, cpt);
		if(strcmp(ptrCategory, "FIN") != 0){
			printf("%d : %s\n", cpt, ptrCategory);
		}	
		cpt++;
	}while(strcmp(ptrCategory, "FIN") != 0);
	
	printf("\n");
	
	/* Liberation de la mémoire */
	free(ptrCategory);
}

void ShowSSCat(xmlDocPtr ptrFic){
	char *ptrCategory, *ptrSSCategory;
	int cpt1, cpt2;
	
	printf("\tNuméro de la catégorie: ");
	scanf("%d", &cpt1);
	ptrCategory = getCategoryById(ptrFic, cpt1);
	
	if(strcmp(ptrCategory, "FIN") != 0){
		printf("\t=> %s\n", ptrCategory);
		cpt2 = 1;
		do{
			ptrSSCategory = getSousCategoryByCategory(ptrFic, cpt1, cpt2);
			if(strcmp(ptrSSCategory, "FIN") != 0){
				printf("\t\t%d-%d : %s\n", cpt1, cpt2, ptrSSCategory);
			}
			cpt2++;
		}while(strcmp(ptrSSCategory, "FIN") != 0);
		free(ptrSSCategory);
	}
	else{
		fprintf(stderr, "\tCatégorie inconnue\n");
	}
	
	printf("\n");
	
	/* Liberation de la mémoire */
	free(ptrCategory);
}

void ShowArt(xmlDocPtr ptrFic){
	char *ptrCategory, *ptrSSCategory, *ptrArticleName, *ptrArticlePrice;
	int cpt1, cpt2, cpt3;
	
	printf("Liste des articles:\n");
	
	printf("\tNuméro de la sous catégorie (IdCat-IdSSCat): ");
	scanf("%d-%d", &cpt1, &cpt2);
	ptrCategory = getCategoryById(ptrFic, cpt1);
	
	if(strcmp(ptrCategory, "FIN") != 0){
		ptrSSCategory = getSousCategoryByCategory(ptrFic, cpt1, cpt2);
		if(strcmp(ptrSSCategory, "FIN") != 0){
			printf("\t=>%s=>%s\n", ptrCategory, ptrSSCategory);
			cpt3 = 1;
			do{
				ptrArticleName = getArticleNameBySousCategory(ptrFic, cpt1, cpt2, cpt3);
				if(strcmp(ptrArticleName, "FIN") != 0){
					ptrArticlePrice = getArticlePriceBySousCategory(ptrFic, cpt1, cpt2, cpt3);
					printf("\t\t%d-%d-%d : %s = %s €\n", cpt1, cpt2, cpt3, ptrArticleName, ptrArticlePrice);
				}
				cpt3++;
			}while(strcmp(ptrArticleName, "FIN") != 0);
			free(ptrArticleName);
			free(ptrArticlePrice);
		}
		else{
			fprintf(stderr, "\tSous catégorie inconnue\n");
		}
		free(ptrSSCategory);
	}
	else{
		fprintf(stderr, "\tCatégorie inconnue\n");
	}
	
	printf("\n");
	
	/* Liberation de la mémoire */
	free(ptrCategory);
}

listeArticles* AddArt(xmlDocPtr ptrFic, listeArticles *ptrLdaChainee){
	char *ptrCategory, *ptrSSCategory, *ptrArticleName, *ptrArticlePrice;
	int tabArt[3], Quantite, Id;
	
	printf("\tNuméro de l'article: ");
	scanf("%d-%d-%d", &tabArt[0], &tabArt[1], &tabArt[2]);
	
	ptrCategory = getCategoryById(ptrFic, tabArt[0]);
	if(strcmp(ptrCategory, "FIN") != 0){
		ptrSSCategory = getSousCategoryByCategory(ptrFic, tabArt[0], tabArt[1]);
		if(strcmp(ptrSSCategory, "FIN") != 0){
			ptrArticleName = getArticleNameBySousCategory(ptrFic, tabArt[0], tabArt[1], tabArt[2]);
			if(strcmp(ptrArticleName, "FIN") != 0){
				printf("\t Quantité: ");
				scanf("%d", &Quantite);
				ptrArticlePrice = getArticlePriceBySousCategory(ptrFic, tabArt[0], tabArt[1], tabArt[2]);
				Id = dejaAjouter(ptrLdaChainee, tabArt);
				if(Id == -1){
					ptrLdaChainee = ajoutNouveauArticle(ptrLdaChainee, tabArt, Quantite);
					if(ptrLdaChainee != NULL){
						printf("\tAjout de %s (%s €)\n", ptrArticleName, ptrArticlePrice);
					}
					else{
						fprintf(stderr, "\tImpossible d'ajouter l'article :(. Re-essayez!\n");
					}
				}
				else{
					mettreAjourQuantite(ptrLdaChainee, Id, Quantite);
					printf("\tAjout de %s (%s €)\n", ptrArticleName, ptrArticlePrice);
				}
				free(ptrArticlePrice);
			}
			else{
				fprintf(stderr, "\tArticle introuvable :(\n");
			}
			free(ptrArticleName);
		}
		else{
			fprintf(stderr, "\tArticle introuvable :(\n");
		}
		free(ptrSSCategory);
	}
	else{
		fprintf(stderr, "\tArticle introuvable :(\n");
	}
	
	printf("\n");

	/* Liberation de la mémoire */
	free(ptrCategory);
	
	return ptrLdaChainee;
}

void ShowMyList(xmlDocPtr ptrFic, listeArticles *ptrLdaChainee){
	listeArticles *ptrCourant;
	int prixArt, prixTotal, nbArt;
	char *ptrArticleName, *ptrArticlePrice;
	
	prixTotal=0;
	nbArt=0;
	
	printf("\tNum : Article \t\tPrix (en euro)\tQuantité\n");
	if(ptrLdaChainee != NULL){
		ptrCourant = ptrLdaChainee;
		while(ptrCourant->ptrSuivant != NULL){
			ptrArticleName = getArticleNameBySousCategory(ptrFic, ptrCourant->Article[0], ptrCourant->Article[1], ptrCourant->Article[2]);
			ptrArticlePrice = getArticlePriceBySousCategory(ptrFic, ptrCourant->Article[0], ptrCourant->Article[1], ptrCourant->Article[2]);
			prixArt = atoi(ptrArticlePrice)*(ptrCourant->Quantite);
			printf("\t%d : %s \t\t%d €\t%d\n", ptrCourant->Id, ptrArticleName, prixArt, ptrCourant->Quantite);
			ptrCourant = (listeArticles *)ptrCourant->ptrSuivant;
			
			prixTotal = prixTotal+prixArt;
			nbArt++;
		}
		ptrArticleName = getArticleNameBySousCategory(ptrFic, ptrCourant->Article[0], ptrCourant->Article[1], ptrCourant->Article[2]);
		ptrArticlePrice = getArticlePriceBySousCategory(ptrFic, ptrCourant->Article[0], ptrCourant->Article[1], ptrCourant->Article[2]);
		prixArt = atoi(ptrArticlePrice)*(ptrCourant->Quantite);
		printf("\t%d : %s \t\t%d €\t%d\n", ptrCourant->Id, ptrArticleName, prixArt, ptrCourant->Quantite);
		prixTotal = prixTotal+prixArt;
		nbArt++;
		free(ptrArticleName);
		free(ptrArticlePrice);
	}
	
	printf("\n\tNombre d'article(s): %d.\n", nbArt);
	printf("\tPrix total: %d €.\n\n", prixTotal);
}

void SaveMyList(xmlDocPtr ptrFic, listeArticles *ptrLdaChainee, char *fichierSVG){
	if(saveListe(ptrLdaChainee, ptrFic, fichierSVG) == VRAI){
		printf("Votre liste a été sauvegardé dans %s\n", fichierSVG);
	}
}

